@@ -459,7 +459,7 @@ def parse_project_stream(stream: TextIO) -> ast.Project:
     ret: ast.Project
     loader = ProjectLoader(stream)
     try:
-        ret = loader.get_single_data()  # type: ignore[no-untyped-call]
+        ret = loader.get_single_data()
         assert isinstance(ret, ast.Project)
         return ret
     finally:
@@ -572,8 +572,9 @@ FlowLoader.add_path_resolver("flow:images", [(dict, "images")])  # type: ignore
 FlowLoader.add_constructor("flow:images", parse_images)  # type: ignore
 
 
-def parse_needs_key(ctor: BaseConstructor, node: yaml.Node) -> IdExpr:
-    tmp = ctor.construct_scalar(node)  # type: ignore[no-untyped-call]
+def parse_needs_key(ctor: BaseConstructor, node: yaml.ScalarNode) -> IdExpr:
+    tmp = ctor.construct_scalar(node)
+    assert tmp is None or isinstance(tmp, str)
     return IdExpr(mark2pos(node.start_mark), mark2pos(node.end_mark), tmp)
 
 
@@ -721,6 +722,12 @@ JOB_ACTION_CALL = {
     "params": None,
 }
 
+JOB_MODULE_CALL = {
+    "module": SimpleStrExpr,
+    "args": SimpleMapping(StrExpr),
+    "params": None,
+}
+
 
 def check_extra(
     node: yaml.Node, dct: Dict[str, Any], ret: Dict[str, Any], name: str
@@ -752,10 +759,13 @@ def select_shells(
     return dct
 
 
-def select_job_or_action(
+def select_job_or_action_or_module(
     ctor: BaseConstructor, node: yaml.MappingNode, dct: Dict[str, Any]
 ) -> Dict[str, Any]:
-    if "action" in dct:
+    if "module" in dct:
+        ret = {k: v for k, v in dct.items() if k in JOB_MODULE_CALL}
+        check_extra(node, dct, ret, "module call")
+    elif "action" in dct:
         ret = {k: v for k, v in dct.items() if k in JOB_ACTION_CALL}
         check_extra(node, dct, ret, "action call")
     else:
@@ -765,7 +775,7 @@ def select_job_or_action(
     return ret
 
 
-JOB_OR_ACTION = {**JOB, **JOB_ACTION_CALL}
+JOB_OR_ACTION_OR_MODULE = {**JOB, **JOB_ACTION_CALL, **JOB_MODULE_CALL}
 
 
 def find_job_type(
@@ -773,25 +783,25 @@ def find_job_type(
     node: yaml.MappingNode,
     res_type: Type[ast.Base],
     arg: Dict[str, Any],
-) -> Union[Type[ast.Job], Type[ast.JobActionCall]]:
-    action = arg.get("action")
-    if action is None:
-        return ast.Job
-    else:
+) -> Union[Type[ast.Job], Type[ast.JobActionCall], Type[ast.JobModuleCall]]:
+    if arg.get("module") is not None:
+        return ast.JobModuleCall
+    if arg.get("action") is not None:
         return ast.JobActionCall
+    return ast.Job
 
 
 def parse_job(
     ctor: BaseConstructor, node: yaml.MappingNode
-) -> Union[ast.Job, ast.JobActionCall]:
+) -> Union[ast.Job, ast.JobActionCall, ast.JobModuleCall]:
     return cast(
-        Union[ast.Job, ast.JobActionCall],
+        Union[ast.Job, ast.JobActionCall, ast.JobModuleCall],
         parse_dict(
             ctor,
             node,
-            JOB_OR_ACTION,
+            JOB_OR_ACTION_OR_MODULE,
             ast.JobBase,
-            preprocess=select_job_or_action,
+            preprocess=select_job_or_action_or_module,
             find_res_type=find_job_type,
         ),
     )
@@ -799,7 +809,7 @@ def parse_job(
 
 def parse_jobs(
     ctor: BaseConstructor, node: yaml.MappingNode
-) -> Dict[str, Union[ast.Job, ast.JobActionCall]]:
+) -> Dict[str, Union[ast.Job, ast.JobActionCall, ast.JobModuleCall]]:
     ret = {}
     for k, v in node.value:
         key = ctor.construct_id(k)
@@ -811,35 +821,43 @@ def parse_jobs(
 FlowLoader.add_path_resolver("flow:jobs", [(dict, "jobs")])  # type: ignore
 FlowLoader.add_constructor("flow:jobs", parse_jobs)  # type: ignore
 
-
-TASK: Mapping[str, Any] = {
+TASK_BASE = {
     "id": OptIdExpr,
     "needs": None,
     "strategy": None,
     "enable": EnableExpr,
     "cache": None,
+}
+
+TASK: Mapping[str, Any] = {
+    **TASK_BASE,
     **EXEC_UNIT,
 }
 
 
 TASK_ACTION_CALL = {
-    "id": OptIdExpr,
-    "needs": None,
-    "strategy": None,
-    "enable": EnableExpr,
-    "cache": None,
+    **TASK_BASE,
     "action": SimpleStrExpr,
     "args": SimpleMapping(StrExpr),
 }
 
-
-TASK_OR_ACTION = {**TASK, **TASK_ACTION_CALL}
+TASK_MODULE_CALL = {
+    **TASK_BASE,
+    "module": SimpleStrExpr,
+    "args": SimpleMapping(StrExpr),
+}
 
 
-def select_task_or_action(
+TASK_OR_ACTION_OR_MODULE = {**TASK, **TASK_ACTION_CALL, **TASK_MODULE_CALL}
+
+
+def select_task_or_action_or_module(
     ctor: BaseConstructor, node: yaml.MappingNode, dct: Dict[str, Any]
 ) -> Dict[str, Any]:
-    if "action" in dct:
+    if "module" in dct:
+        ret = {k: v for k, v in dct.items() if k in TASK_MODULE_CALL}
+        check_extra(node, dct, ret, "module call")
+    elif "action" in dct:
         ret = {k: v for k, v in dct.items() if k in TASK_ACTION_CALL}
         check_extra(node, dct, ret, "action call")
     else:
@@ -854,21 +872,21 @@ def find_task_type(
     node: yaml.MappingNode,
     res_type: Type[ast.Base],
     arg: Dict[str, Any],
-) -> Union[Type[ast.Task], Type[ast.TaskActionCall]]:
-    action = arg.get("action")
-    if action is None:
-        return ast.Task
-    else:
+) -> Union[Type[ast.Task], Type[ast.TaskActionCall], Type[ast.TaskModuleCall]]:
+    if arg.get("module") is not None:
+        return ast.TaskModuleCall
+    if arg.get("action") is not None:
         return ast.TaskActionCall
+    return ast.Task
 
 
 def parse_task(ctor: BaseConstructor, node: yaml.MappingNode) -> ast.Base:
     return parse_dict(
         ctor,
         node,
-        TASK_OR_ACTION,
+        TASK_OR_ACTION_OR_MODULE,
         ast.Base,
-        preprocess=select_task_or_action,
+        preprocess=select_task_or_action_or_module,
         find_res_type=find_task_type,
     )
 
@@ -942,7 +960,7 @@ FLOW = {
 }
 
 
-PARAMS = {"default": SimpleOptStrExpr, "descr": SimpleOptStrExpr}
+PARAMS = {"default": OptStrExpr, "descr": OptStrExpr}
 
 
 def parse_param(ctor: BaseConstructor, node: yaml.MappingNode) -> ast.Param:
@@ -960,8 +978,8 @@ def parse_params(ctor: BaseConstructor, node: yaml.MappingNode) -> Dict[str, ast
             ret[key] = ast.Param(
                 _start=start,
                 _end=end,
-                default=SimpleOptStrExpr(start, end, default),
-                descr=SimpleOptStrExpr(start, end, None),
+                default=OptStrExpr(start, end, default),
+                descr=OptStrExpr(start, end, None),
             )
         else:
             arg = parse_param(ctor, v)
@@ -1019,7 +1037,7 @@ FlowLoader.add_constructor("flow:main", parse_flow_main)  # type: ignore
 def parse_live_stream(stream: TextIO) -> ast.LiveFlow:
     loader = FlowLoader(stream, kind=ast.FlowKind.LIVE)
     try:
-        ret = loader.get_single_data()  # type: ignore[no-untyped-call]
+        ret = loader.get_single_data()
         assert isinstance(ret, ast.LiveFlow)
         assert ret.kind == ast.FlowKind.LIVE
         return ret
@@ -1036,7 +1054,7 @@ def parse_live(workspace: LocalPath, config_file: LocalPath) -> ast.LiveFlow:
 def parse_batch_stream(stream: TextIO) -> ast.BatchFlow:
     loader = FlowLoader(stream, kind=ast.FlowKind.BATCH)
     try:
-        ret = loader.get_single_data()  # type: ignore[no-untyped-call]
+        ret = loader.get_single_data()
         assert isinstance(ret, ast.BatchFlow)
         assert ret.kind == ast.FlowKind.BATCH
         return ret
@@ -1417,8 +1435,8 @@ def parse_meta_main(ctor: BaseConstructor, node: yaml.MappingNode) -> Mapping[st
         )
     ret = {}
     for k, v in node.value:
-        key = str(ctor.construct_scalar(k))  # type: ignore[no-untyped-call]
-        value = str(ctor.construct_scalar(v))  # type: ignore[no-untyped-call]
+        key = str(ctor.construct_scalar(k))
+        value = str(ctor.construct_scalar(v))
         ret[key] = value
     return ret
 
@@ -1431,7 +1449,7 @@ def parse_action_stream(stream: TextIO) -> ast.BaseAction:
     ret: ast.Project
     loader = ActionLoader(stream)
     try:
-        ret = loader.get_single_data()  # type: ignore[no-untyped-call]
+        ret = loader.get_single_data()
         assert isinstance(ret, ast.BaseAction)
         return ret
     finally:
@@ -1446,6 +1464,6 @@ def parse_action(action_file: LocalPath) -> ast.BaseAction:
 
 def parse_bake_meta(meta_file: LocalPath) -> Mapping[str, str]:
     with meta_file.open() as f:
-        result = BakeMetaLoader(f).get_single_data()  # type: ignore[no-untyped-call]
+        result = BakeMetaLoader(f).get_single_data()
         assert isinstance(result, dict)
         return result
