@@ -5,7 +5,7 @@ import pytest
 import shutil
 import sys
 from collections import defaultdict
-from datetime import datetime
+from datetime import datetime, timedelta
 from neuro_sdk import (
     Client,
     Container,
@@ -57,7 +57,7 @@ from neuro_flow.storage import (
     Storage,
     _Unset,
 )
-from neuro_flow.types import ImageStatus, LocalPath, TaskStatus
+from neuro_flow.types import FullID, ImageStatus, LocalPath, TaskStatus
 
 
 MakeBatchRunner = Callable[[Path], Awaitable[BatchRunner]]
@@ -318,17 +318,15 @@ class MockStorage(FSStorage):
     async def create_bake_image(
         self,
         bake: Bake,
-        prefix: Tuple[str, ...],
-        yaml_id: str,
+        yaml_defs: Sequence[FullID],
         ref: str,
         context_on_storage: Optional[URL],
         dockerfile_rel: Optional[str],
     ) -> BakeImage:
         image = BakeImage(
             id=f"image-{self._index}",
-            prefix=prefix,
             ref=ref,
-            yaml_id=yaml_id,
+            yaml_defs=yaml_defs,
             context_on_storage=context_on_storage,
             dockerfile_rel=dockerfile_rel,
             status=ImageStatus.PENDING,
@@ -805,6 +803,34 @@ async def test_batch_with_action_ok(
     await executor_task
 
 
+async def test_batch_with_module_ok(
+    jobs_mock: JobsMock,
+    assets: Path,
+    run_executor: Callable[[Path, str], Awaitable[None]],
+) -> None:
+    executor_task = asyncio.ensure_future(
+        run_executor(assets / "batch_module", "batch-module-call")
+    )
+    task = await jobs_mock.get_task("test.task-1")
+    assert task.container.command
+    assert "batch_module_call" in task.container.command
+    assert task.container.env.get("TEST") == "test_value"
+    assert "test-tag" in task.tags
+    assert task.preset_name == "test-preset"
+    assert task.life_span == timedelta(days=2).total_seconds()
+    assert task.schedule_timeout == timedelta(minutes=60).total_seconds()
+    assert task.container.working_dir == "/some/dir"
+    assert task.container.volumes[0].container_path == "/volume"
+    await jobs_mock.mark_done("test.task-1", b"::set-output name=task1::Task 1 val 1")
+
+    task = await jobs_mock.get_task("test.task-2")
+    assert task.container.command
+    assert "batch_module_call" in task.container.command
+    assert task.container.env.get("TEST") == "test_value"
+    await jobs_mock.mark_done("test.task-2", b"::set-output name=task2::Task 2 value 2")
+    await executor_task
+
+
 @pytest.mark.parametrize(
     "batch_name,vars",
     [
